<?php

namespace Drupal\aws_cloud\Controller\Ec2;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\cloud\Traits\AccessCheckTrait;

/**
 * Access controller for the Volume entity.
 *
 * @see \Drupal\aws_cloud\Entity\Ec2\Volume\Entity\Volume.
 */
class VolumeAccessControlHandler extends EntityAccessControlHandler {

  use AccessCheckTrait;

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {

    switch ($operation) {
      case 'view':
        return $this->allowedIfCanAccessCloudConfigWithOwner(
          $entity,
          $account,
          'view own aws cloud volume',
          'view any aws cloud volume'
        );

      case 'update':
      case 'edit':
        return $this->allowedIfCanAccessCloudConfigWithOwner(
          $entity,
          $account,
          'edit own aws cloud volume',
          'edit any aws cloud volume'
        );

      case 'delete':
        return $this->allowedIfCanAccessCloudConfigWithOwner(
          $entity,
          $account,
          'delete own aws cloud volume',
          'delete any aws cloud volume'
        );

      case 'attach':
        if ($entity->getState() == 'available') {
          return $this->allowedIfCanAccessCloudConfigWithOwner(
            $entity,
            $account,
            'edit own aws cloud volume',
            'edit any aws cloud volume'
          );
        }
        break;

      case 'detach':
        if ($entity->getState() == 'in-use') {
          return $this->allowedIfCanAccessCloudConfigWithOwner(
            $entity,
            $account,
            'edit own aws cloud volume',
            'edit any aws cloud volume'
          );
        }
        break;
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

}
