<?php

namespace Drupal\k8s\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Lock\LockBackendInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslationInterface;

use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;

/**
 * K8sServiceMock service interacts with the K8s API.
 */
class K8sServiceMock extends K8sService {

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs a new K8sService entity.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   A logger instance.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A configuration factory.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager.
   * @param \Drupal\core\Field\FieldTypePluginManagerInterface $field_type_plugin_manager
   *   The field type plugin manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Lock\LockBackendInterface $lock
   *   The lock interface.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    ConfigFactoryInterface $config_factory,
    Messenger $messenger,
    TranslationInterface $string_translation,
    AccountInterface $current_user,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    FieldTypePluginManagerInterface $field_type_plugin_manager,
    EntityFieldManagerInterface $entity_field_manager,
    LockBackendInterface $lock
  ) {
    parent::__construct(
      $entity_type_manager,
      $logger_factory,
      $config_factory,
      $messenger,
      $string_translation,
      $current_user,
      $cloud_config_plugin_manager,
      $field_type_plugin_manager,
      $entity_field_manager,
      $lock
    );

    $this->config = $config_factory->get('k8s.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getNodes(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getNodes'];
  }

  /**
   * {@inheritdoc}
   */
  public function getNamespaces(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getNamespaces'];
  }

  /**
   * {@inheritdoc}
   */
  public function createNamespace(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createNamespace'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateNamespace(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateNamespace'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteNamespace(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteNamespace'];
  }

  /**
   * {@inheritdoc}
   */
  public function getPods(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getPods'];
  }

  /**
   * {@inheritdoc}
   */
  public function createPod($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createPod'];
  }

  /**
   * {@inheritdoc}
   */
  public function updatePod($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updatePod'];
  }

  /**
   * {@inheritdoc}
   */
  public function deletePod($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deletePod'];
  }

  /**
   * {@inheritdoc}
   */
  public function getPodLogs($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getPodLogs'];
  }

  /**
   * {@inheritdoc}
   */
  public function getDeployments(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getDeployments'];
  }

  /**
   * {@inheritdoc}
   */
  public function createDeployment($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createDeployment'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateDeployment($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateDeployment'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDeployment($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteDeployment'];
  }

  /**
   * {@inheritdoc}
   */
  public function getReplicaSets(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getReplicaSets'];
  }

  /**
   * {@inheritdoc}
   */
  public function createReplicaSet($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createReplicaSet'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateReplicaSet($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateReplicaSet'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteReplicaSet($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteReplicaSet'];
  }

  /**
   * {@inheritdoc}
   */
  public function getServices(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getServices'];
  }

  /**
   * {@inheritdoc}
   */
  public function createService($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createService'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateService($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateService'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteService($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteService'];
  }

  /**
   * {@inheritdoc}
   */
  public function getCronJobs(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getCronJobs'];
  }

  /**
   * {@inheritdoc}
   */
  public function createCronJob($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createCronJob'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateCronJob($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateCronJob'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteCronJob($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteCronJob'];
  }

  /**
   * {@inheritdoc}
   */
  public function getJobs(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getJobs'];
  }

  /**
   * {@inheritdoc}
   */
  public function createJob($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createJob'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateJob($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateJob'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteJob($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteJob'];
  }

  /**
   * {@inheritdoc}
   */
  public function getResourceQuotas(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getResourceQuotas'];
  }

  /**
   * {@inheritdoc}
   */
  public function createResourceQuota($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createResourceQuota'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateResourceQuota($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateResourceQuota'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteResourceQuota($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteResourceQuota'];
  }

  /**
   * {@inheritdoc}
   */
  public function getLimitRanges(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getLimitRanges'];
  }

  /**
   * {@inheritdoc}
   */
  public function createLimitRange($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createLimitRange'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateLimitRange($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateLimitRange'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteLimitRange($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteLimitRange'];
  }

  /**
   * {@inheritdoc}
   */
  public function getSecrets(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getSecrets'];
  }

  /**
   * {@inheritdoc}
   */
  public function createSecret($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createSecret'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateSecret($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateSecret'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteSecret($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteSecret'];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigMaps(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getConfigMaps'];
  }

  /**
   * {@inheritdoc}
   */
  public function createConfigMap($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createConfigMap'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateConfigMap($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateConfigMap'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteConfigMap($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteConfigMap'];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetricsPods(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getMetricsPods'];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetricsNodes(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getMetricsNodes'];
  }

  /**
   * {@inheritdoc}
   */
  public function getNetworkPolicies(array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['getNetworkPolicies'];
  }

  /**
   * {@inheritdoc}
   */
  public function createNetworkPolicy($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['createNetworkPolicy'];
  }

  /**
   * {@inheritdoc}
   */
  public function updateNetworkPolicy($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['updateNetworkPolicy'];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteNetworkPolicy($namespace, array $params = []) {
    return json_decode($this->config->get('k8s_mock_data'), TRUE)['deleteNetworkPolicy'];
  }

}
