<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s deployment.
 *
 * @group k8s
 */
class K8sDeploymentTest extends K8sTestCase {

  const K8S_DEPLOYMENT_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];

    return [
      'list k8s deployment',
      'view k8s deployment',
      'edit k8s deployment',
      'add k8s deployment',
      'delete k8s deployment',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Deployment.
   */
  public function testDeployment() {

    $cloud_context = $this->cloudContext;

    // List Deployment for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/deployment");
    $this->assertResponse(200);

    // Add a new Deployment.
    $add = $this->createDeploymentTestFormData(self::K8S_DEPLOYMENT_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_DEPLOYMENT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addDeploymentMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/deployment/add",
        $add[$i]['post_data'],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t('The Kubernetes Deployment "@name" has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/deployment");
      $this->assertResponse(200);
      $this->assertText($add[$i]['name']);
    }

    // Edit a Deployment.
    $edit = $this->createDeploymentTestFormData(self::K8S_DEPLOYMENT_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_DEPLOYMENT_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateDeploymentMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/deployment/$num/edit",
        $edit[$i]['post_data'],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Deployment "@name" has been saved.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Deployment.
    for ($i = 0; $i < self::K8S_DEPLOYMENT_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteDeploymentMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/deployment/$num/delete",
        [],
        t('Delete')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Deployment "@name" has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/deployment");
      $this->assertResponse(200);
      $this->assertNoText($add[$i]['name']);
    }
  }

  /**
   * Tests deleting deployments with bulk operation.
   */
  public function testDeploymentBulk() {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::K8S_DEPLOYMENT_REPEAT_COUNT; $i++) {
      // Create Deployments.
      $deployments = $this->createDeploymentsRandomTestFormData($this->namespace);
      $entities = [];
      foreach ($deployments as $deployment) {
        $entities[] = $this->createDeploymentTestEntity($deployment);
      }
      $this->deleteDeploymentMockData($deployments[0]);
      $this->doTestEntityBulk('deployment', $entities);
    }
  }

}
