<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s namespace.
 *
 * @group k8s
 */
class K8sNamespaceTest extends K8sTestCase {

  const K8S_NAMESPACE_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list k8s namespace',
      'view k8s namespace',
      'edit k8s namespace',
      'add k8s namespace',
      'delete k8s namespace',
    ];
  }

  /**
   * Tests CRUD for Namespace.
   */
  public function testNamespace() {

    $cloud_context = $this->cloudContext;

    // List Namespace for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
    $this->assertResponse(200);

    // Add a new Namespace.
    $add = $this->createNamespaceTestFormData(self::K8S_NAMESPACE_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_NAMESPACE_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addNamespaceMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/namespace/add",
        $add[$i],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t('The Kubernetes Namespace "@name" has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
      $this->assertResponse(200);
      $this->assertText($add[$i]['name']);
    }

    // Edit a Namespace.
    $edit = $this->createNamespaceTestFormData(self::K8S_NAMESPACE_REPEAT_COUNT);
    for ($i = 0; $i < self::K8S_NAMESPACE_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateNamespaceMockData($edit[$i]);

      unset($edit[$i]['name']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/namespace/$num/edit",
        $edit[$i],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Namespace "@name" has been saved.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure labels.
      $this->drupalGet("/clouds/k8s/$cloud_context/namespace/$num");
      $this->assertResponse(200);
      $this->assertText($edit[$i]['labels[0][item_key]']);
      $this->assertText($edit[$i]['labels[0][item_value]']);
    }

    // Delete Namespace.
    for ($i = 0; $i < self::K8S_NAMESPACE_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteNamespaceMockData($add[$i]);

      $this->drupalGet("/clouds/k8s/$cloud_context/namespace/$num/delete");
      $this->assertResponse(200);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/namespace/$num/delete",
        [],
        t('Delete')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Namespace "@name" has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/namespace");
      $this->assertResponse(200);
      $this->assertNoText($add[$i]['name']);
    }
  }

  /**
   * Tests deleting namespaces with bulk operation.
   */
  public function testNamespaceBulk() {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::K8S_NAMESPACE_REPEAT_COUNT; $i++) {
      // Create Namespaces.
      $namespaces = $this->createNamespacesRandomTestFormData();
      $entities = [];
      foreach ($namespaces as $namespace) {
        $entities[] = $this->createNamespaceTestEntity($namespace);
      }
      $this->deleteNamespaceMockData($namespaces[0]);
      $this->doTestEntityBulk('namespace', $entities);
    }
  }

}
