<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s resource quota.
 *
 * @group k8s
 */
class K8sResourceQuotaTest extends K8sTestCase {

  const K8S_RESOURCE_QUOTA_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];

    return [
      'list k8s resource quota',
      'view k8s resource quota',
      'edit k8s resource quota',
      'add k8s resource quota',
      'delete k8s resource quota',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Resource Quota.
   */
  public function testResourceQuota() {

    $cloud_context = $this->cloudContext;

    // List Resource Quota for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/resource_quota");
    $this->assertResponse(200);

    // Add a new Resource Quota.
    $add = $this->createResourceQuotaTestFormData(self::K8S_RESOURCE_QUOTA_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_RESOURCE_QUOTA_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addResourceQuotaMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/resource_quota/add",
        $add[$i]['post_data'],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t('The Kubernetes Resource Quota "@name" has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/resource_quota");
      $this->assertResponse(200);
      $this->assertText($add[$i]['name']);
    }

    // Edit a Resource Quota.
    $edit = $this->createResourceQuotaTestFormData(self::K8S_RESOURCE_QUOTA_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_RESOURCE_QUOTA_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateResourceQuotaMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/resource_quota/$num/edit",
        $edit[$i]['post_data'],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Resource Quota "@name" has been saved.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Resource Quota.
    for ($i = 0; $i < self::K8S_RESOURCE_QUOTA_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteResourceQuotaMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/resource_quota/$num/delete",
        [],
        t('Delete')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Resource Quota "@name" has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/resource_quota");
      $this->assertResponse(200);
      $this->assertNoText($add[$i]['name']);
    }
  }

  /**
   * Tests deleting resource quotas with bulk operation.
   */
  public function testResourceQuotaBulk() {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::K8S_RESOURCE_QUOTA_REPEAT_COUNT; $i++) {
      // Create Resource Quotas.
      $resource_quotas = $this->createResourceQuotasRandomTestFormData($this->namespace);
      $entities = [];
      foreach ($resource_quotas as $resource_quota) {
        $entities[] = $this->createResourceQuotaTestEntity($resource_quota);
      }
      $this->deleteResourceQuotaMockData($resource_quotas[0]);
      $this->doTestEntityBulk('resource_quota', $entities);
    }
  }

}
