<?php

namespace Drupal\openstack\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\user\EntityOwnerInterface;

/**
 * Provides an interface for defining OpenStack Cloud Instance entities.
 *
 * @ingroup openstack
 */
interface InstanceInterface extends ContentEntityInterface, EntityChangedInterface, EntityOwnerInterface {

  /**
   * Add get/set methods for your configuration properties here.
   */

  /**
   * Gets the OpenStack Cloud Instance name.
   *
   * @return string
   *   Name of the OpenStack Cloud Instance.
   */
  public function getName();

  /**
   * Sets the OpenStack Cloud Instance name.
   *
   * @param string $name
   *   The OpenStack Cloud Instance name.
   *
   * @return \Drupal\openstack\Entity\InstanceInterface
   *   The called OpenStack Cloud Instance entity.
   */
  public function setName($name);

  /**
   * Gets the OpenStack Cloud Instance creation timestamp.
   *
   * @return int
   *   Creation timestamp of the OpenStack Cloud Instance.
   */
  public function getCreatedTime();

  /**
   * Sets the OpenStack Cloud Instance creation timestamp.
   *
   * @param int $timestamp
   *   The OpenStack Cloud Instance creation timestamp.
   *
   * @return \Drupal\openstack\Entity\InstanceInterface
   *   The called OpenStack Cloud Instance entity.
   */
  public function setCreatedTime($timestamp);

  /**
   * {@inheritdoc}
   */
  public function setInstanceId($instance_id = '');

  /**
   * {@inheritdoc}
   */
  public function getInstanceId();

  /**
   * {@inheritdoc}
   */
  public function setInstanceState($state = '');

  /**
   * {@inheritdoc}
   */
  public function getInstanceState();

  /**
   * {@inheritdoc}
   */
  public function setInstanceType($type = '');

  /**
   * {@inheritdoc}
   */
  public function getInstanceType();

  /**
   * {@inheritdoc}
   */
  public function setReservation($reservation = '');

  /**
   * {@inheritdoc}
   */
  public function getReservation();

  /**
   * {@inheritdoc}
   */
  public function setImageId($image_id = '');

  /**
   * {@inheritdoc}
   */
  public function getImageId();

  /**
   * {@inheritdoc}
   */
  public function setAccountId($account_id = '');

  /**
   * {@inheritdoc}
   */
  public function getAccountId();

  /**
   * {@inheritdoc}
   */
  public function setLaunchTime($launch_time = '');

  /**
   * {@inheritdoc}
   */
  public function getLaunchTime();

  /**
   * {@inheritdoc}
   */
  public function getPublicDns();

  /**
   * {@inheritdoc}
   */
  public function setPublicDns($dns = '');

  /**
   * {@inheritdoc}
   */
  public function getPrivateDns();

  /**
   * {@inheritdoc}
   */
  public function setPrivateDns($private_dns = '');

  /**
   * {@inheritdoc}
   */
  public function getPrivateIps();

  /**
   * {@inheritdoc}
   */
  public function setPrivateIps($private_ips = '');

  /**
   * {@inheritdoc}
   */
  public function getSecurityGroups();

  /**
   * {@inheritdoc}
   */
  public function setSecurityGroups(array $security_groups);

  /**
   * {@inheritdoc}
   */
  public function setKeyPairName($key_pair = '');

  /**
   * {@inheritdoc}
   */
  public function getKeyPairName();

  /**
   * {@inheritdoc}
   */
  public function setAvailabilityZone($zone = '');

  /**
   * {@inheritdoc}
   */
  public function getAvailabilityZone();

  /**
   * {@inheritdoc}
   */
  public function setRootDeviceType($type = '');

  /**
   * {@inheritdoc}
   */
  public function getRootDeviceType();

  /**
   * {@inheritdoc}
   */
  public function setRootDevice($device = '');

  /**
   * {@inheritdoc}
   */
  public function getRootDevice();

  /**
   * {@inheritdoc}
   */
  public function setAmiLaunchIndex($launch_index = '');

  /**
   * {@inheritdoc}
   */
  public function getAmiLaunchIndex();

}
