<?php

namespace Drupal\openstack;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the OpenStack Cloud Instance entity.
 *
 * @see \Drupal\openstack\Entity\Instance.
 */
class InstanceAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\openstack\Entity\InstanceInterface $entity */
    switch ($operation) {
      case 'view':
        return AccessResult::allowedIfHasPermission($account, 'view published openstack cloud instances');

      case 'edit':
      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit openstack cloud instances');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete openstack cloud instances');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add openstack cloud instances');
  }

}
